@@ -26,7 +26,7 @@ from macrometa_target_snowflake.file_formats import csv
 from macrometa_target_snowflake.file_formats import parquet
 from macrometa_target_snowflake import stream_utils
 
-from macrometa_target_snowflake.db_sync import DbSync
+from macrometa_target_snowflake.db_sync import DbSync, create_private_key_file, delete_private_key_file
 from macrometa_target_snowflake.file_format import FileFormatTypes
 from macrometa_target_snowflake.exceptions import (
     RecordValidationException,
@@ -614,7 +614,7 @@ class SnowflakeTargetConnector(C8Connector):
             ConfigProperty('user', 'Username', ConfigAttributeType.STRING, True, False,
                            description='Snowflake user name.',
                            placeholder_value='my_username'),
-            ConfigProperty('password', 'Password', ConfigAttributeType.STRING, True, False,
+            ConfigProperty('password', 'Password', ConfigAttributeType.STRING, False, False,
                            description='Snowflake password.',
                            placeholder_value='my_password'),
             ConfigProperty('dbname', 'Database', ConfigAttributeType.STRING, True, False,
@@ -649,7 +649,13 @@ class SnowflakeTargetConnector(C8Connector):
                            placeholder_value='SELECT'),
             ConfigProperty('hard_delete', 'Hard Delete', ConfigAttributeType.BOOLEAN, False, False,
                            description='Indicates whether to perform a hard delete on records.',
-                           default_value='false')
+                           default_value='false'),
+            ConfigProperty('private_key_path', 'Private Key', ConfigAttributeType.STRING, False, False,
+                           description='A private key used for authenticating using Key Pair authentication instead of user/pass. At the moment, only PEM format is supported.',
+                           placeholder_value='my_private_key'),
+            ConfigProperty('private_key_passphrase', 'Private Key Passphrase', ConfigAttributeType.STRING, False, False,
+                           description='The private key passphrase used for authenticating using Key Pair authentication instead of user/pass.',
+                           placeholder_value='my_private_key_passphrase'),
         ]
 
     def capabilities(self) -> list[str]:
@@ -671,12 +677,22 @@ def main():
     else:
         config = {}
 
-    # Init columns cache
-    table_cache, file_format_type = get_snowflake_statics(config)
+    # Create private key file
+    config = create_private_key_file(config)
 
-    # Consume singer messages
-    singer_messages = io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8')
-    persist_lines(config, singer_messages, table_cache, file_format_type)
+    try:
+        # Init columns cache
+        table_cache, file_format_type = get_snowflake_statics(config)
+
+        # Consume singer messages
+        singer_messages = io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8')
+        persist_lines(config, singer_messages, table_cache, file_format_type)
+    except Exception as e:
+        delete_private_key_file(config)
+        raise e
+
+    # Delete private key file
+    delete_private_key_file(config)
 
     LOGGER.debug("Exiting normally")
 
