@@ -3,9 +3,13 @@ import sys
 import snowflake.connector
 import re
 import time
+import singer
+import uuid
+from pathlib import Path
+from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives import serialization
 
 from typing import List, Dict, Union, Tuple, Set
-from singer import get_logger
 from macrometa_target_snowflake import flattening
 from macrometa_target_snowflake import stream_utils
 from macrometa_target_snowflake.file_format import FileFormat, FileFormatTypes
@@ -14,6 +18,8 @@ from macrometa_target_snowflake.exceptions import TooManyRecordsException, Prima
 from macrometa_target_snowflake.upload_clients.s3_upload_client import S3UploadClient
 from macrometa_target_snowflake.upload_clients.snowflake_upload_client import SnowflakeUploadClient
 
+LOGGER = singer.get_logger('macrometa_target_snowflake')
+
 
 def validate_config(config):
     """Validate configuration"""
@@ -22,7 +28,6 @@ def validate_config(config):
         'account',
         'dbname',
         'user',
-        'password',
         'warehouse',
         's3_bucket',
         'stage',
@@ -33,13 +38,17 @@ def validate_config(config):
         'account',
         'dbname',
         'user',
-        'password',
         'warehouse',
         'file_format'
     ]
 
     required_config_keys = []
 
+    possible_authentication_keys = [
+        'password',
+        'private_key_path'
+    ]
+
     # Use external stages if both s3_bucket and stage defined
     if config.get('s3_bucket', None) and config.get('stage', None):
         required_config_keys = s3_required_config_keys
@@ -72,6 +81,10 @@ def validate_config(config):
         errors.append(
             'Archive load files option can be used only with external s3 stages. Please define s3_bucket.')
 
+    if not any(config.get(k, None) for k in possible_authentication_keys):
+        errors.append(
+            f'Required authentication key missing. Existing methods: {",".join(possible_authentication_keys)}')
+
     return errors
 
 
@@ -171,6 +184,56 @@ def create_query_tag(query_tag_pattern: str, database: str = None, schema: str =
     return query_tag
 
 
+# PRIVATE KEY PATH
+def create_private_key_file(config: Dict) -> Dict:
+    path_uuid = uuid.uuid4().hex
+    try:
+        if config.get('private_key_path'):
+            path = f"/opt/snowflake/{path_uuid}/rsa_key.p8"
+            private_key = Path(path)
+            private_key.parent.mkdir(exist_ok=True, parents=True)
+            private_key.write_text(
+                create_private_key_string(config['private_key_path']))
+            config['private_key_path'] = path
+            LOGGER.info(f"Private key file created at: {path}")
+
+    except Exception as e:
+        LOGGER.warn(
+            f'Failed to create private key: /opt/snowflake/{path_uuid}/rsa_key.p8')
+
+    return config
+
+
+def delete_private_key_file(config: Dict) -> None:
+    try:
+        private_key = None
+        if config.get('private_key_path'):
+            path = config['private_key_path']
+            private_key = Path(path)
+            config['private_key_path'] = private_key.read_text()
+            private_key.unlink()
+            LOGGER.info(f"Private key file deleted from: {path}")
+
+        if private_key is not None:
+            private_key.parent.rmdir()
+    except Exception as e:
+        LOGGER.warn(f"Failed to delete private key file: {e}")
+
+
+def create_private_key_string(private_key_string: str) -> str:
+    private_key_list = []
+    split_string = private_key_string.split("-----")
+    if len(split_string) < 4:
+        raise Exception("Invalid PEM format for private key.")
+    for i in range(len(split_string)):
+        if ((i % 2) == 1):
+            private_key_list.extend(("-----", split_string[i], "-----"))
+        else:
+            private_key_list.append(split_string[i].replace(' ', '\n'))
+
+    return ''.join(private_key_list)
+
+
 # pylint: disable=too-many-public-methods,too-many-instance-attributes
 class DbSync:
     """DbSync class"""
@@ -199,7 +262,7 @@ class DbSync:
         self.table_cache = table_cache
 
         # logger to be used across the class's methods
-        self.logger = get_logger('macrometa_target_snowflake')
+        self.logger = singer.get_logger('macrometa_target_snowflake')
 
         # Validate connection configuration
         config_errors = validate_config(connection_config)
@@ -272,6 +335,32 @@ class DbSync:
         else:
             self.upload_client = SnowflakeUploadClient(connection_config, self)
 
+    def get_private_key(self):
+        """
+        Get private key from the right location
+        """
+        if self.connection_config.get('private_key_path'):
+            try:
+                encoded_passphrase = self.connection_config['private_key_passphrase'].encode(
+                )
+            except KeyError:
+                encoded_passphrase = None
+
+            with open(self.connection_config['private_key_path'], 'rb') as key:
+                p_key = serialization.load_pem_private_key(
+                    key.read(),
+                    password=encoded_passphrase,
+                    backend=default_backend()
+                )
+
+            pkb = p_key.private_bytes(
+                encoding=serialization.Encoding.DER,
+                format=serialization.PrivateFormat.PKCS8,
+                encryption_algorithm=serialization.NoEncryption())
+            return pkb
+
+        return None
+
     def open_connection(self):
         """Open snowflake connection"""
         stream = None
@@ -280,7 +369,8 @@ class DbSync:
 
         return snowflake.connector.connect(
             user=self.connection_config['user'],
-            password=self.connection_config['password'],
+            password=self.connection_config.get('password'),
+            private_key=self.get_private_key(),
             account=self.connection_config['account'],
             database=self.connection_config['dbname'],
             warehouse=self.connection_config['warehouse'],
